@@ -14,7 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -45,9 +45,9 @@
 #endif
 
 #include "agent.h"
-#include "simple-pwquery.h"
-#include "i18n.h"
-#include "sysutils.h"
+#include "../common/simple-pwquery.h"
+#include "../common/i18n.h"
+#include "../common/sysutils.h"
 #include "../common/init.h"
 
 
@@ -111,10 +111,6 @@ my_strusage (int level)
 
 
 
-/* Include the implementation of map_spwq_error.  */
-MAP_SPWQ_ERROR_IMPL
-
-
 static void
 preset_passphrase (const char *keygrip)
 {
@@ -170,7 +166,7 @@ preset_passphrase (const char *keygrip)
   if (!opt_passphrase)
     wipememory (passphrase, sizeof (passphrase));
 
-  rc = map_spwq_error (simple_query (line));
+  rc = simple_query (line);
   if (rc)
     {
       log_error ("caching passphrase failed: %s\n", gpg_strerror (rc));
@@ -192,7 +188,7 @@ forget_passphrase (const char *keygrip)
   if (rc < 0)
     rc = gpg_error_from_syserror ();
   else
-    rc = map_spwq_error (simple_query (line));
+    rc = simple_query (line);
   if (rc)
     {
       log_error ("clearing passphrase failed: %s\n", gpg_strerror (rc));
@@ -212,7 +208,7 @@ main (int argc, char **argv)
 
   early_system_init ();
   set_strusage (my_strusage);
-  log_set_prefix ("gpg-preset-passphrase", 1);
+  log_set_prefix ("gpg-preset-passphrase", GPGRT_LOG_WITH_PREFIX);
 
   /* Make sure that our subsystems are ready.  */
   i18n_init ();
@@ -243,7 +239,7 @@ main (int argc, char **argv)
   else
     usage (1);
 
-  /* Tell simple-pwquery about the the standard socket name.  */
+  /* Tell simple-pwquery about the standard socket name.  */
   {
     char *tmp = make_filename (gnupg_socketdir (), GPG_AGENT_SOCK_NAME, NULL);
     simple_pw_set_socket (tmp);
