@@ -25,7 +25,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -232,35 +232,39 @@ copy_buffer_do_copy (struct copy_buffer *c, estream_t source, estream_t sink)
   if (c->nread == 0)
     {
       c->writep = c->buffer;
-      err = es_read (source, c->buffer, sizeof c->buffer, &c->nread);
-      if (err)
+      if (es_read (source, c->buffer, sizeof c->buffer, &c->nread))
         {
-          if (errno == EAGAIN)
+          err = my_error_from_syserror ();
+          if (gpg_err_code (err) == GPG_ERR_EAGAIN)
             return 0;	/* We will just retry next time.  */
 
-          return my_error_from_syserror ();
+          return err;
         }
 
-      assert (c->nread <= sizeof c->buffer);
+      log_assert (c->nread <= sizeof c->buffer);
     }
 
   if (c->nread == 0)
     return 0;	/* Done copying.  */
 
+  nwritten = 0;
+  if (sink && es_write (sink, c->writep, c->nread, &nwritten))
+    err = my_error_from_syserror ();
+  else
+    err = 0;
 
-  err = sink? es_write (sink, c->writep, c->nread, &nwritten) : 0;
-  if (err)
-    {
-      if (errno == EAGAIN)
-        return 0;	/* We will just retry next time.  */
-
-      return my_error_from_syserror ();
-    }
-
-  assert (nwritten <= c->nread);
+  log_assert (nwritten <= c->nread);
   c->writep += nwritten;
   c->nread -= nwritten;
-  assert (c->writep - c->buffer <= sizeof c->buffer);
+  log_assert (c->writep - c->buffer <= sizeof c->buffer);
+
+  if (err)
+    {
+      if (gpg_err_code (err) == GPG_ERR_EAGAIN)
+        return 0;	/* We will just retry next time.  */
+
+      return err;
+    }
 
   if (sink && es_fflush (sink) && errno != EAGAIN)
     err = my_error_from_syserror ();
@@ -273,16 +277,24 @@ copy_buffer_do_copy (struct copy_buffer *c, estream_t source, estream_t sink)
 static gpg_error_t
 copy_buffer_flush (struct copy_buffer *c, estream_t sink)
 {
-  gpg_error_t err;
+  gpg_error_t err = 0;
+  size_t nwritten = 0;
 
-  while (c->nread > 0)
-    {
-      err = copy_buffer_do_copy (c, NULL, sink);
-      if (err)
-        return err;
-    }
+  if (es_write (sink, c->writep, c->nread, &nwritten))
+    err = my_error_from_syserror ();
 
-  return 0;
+  log_assert (nwritten <= c->nread);
+  c->writep += nwritten;
+  c->nread -= nwritten;
+  log_assert (c->writep - c->buffer <= sizeof c->buffer);
+
+  if (err)
+    return err;
+
+  if (es_fflush (sink))
+    err = my_error_from_syserror ();
+
+  return err;
 }
 
 
@@ -372,7 +384,12 @@ gnupg_exec_tool_stream (const char *pgmname, const char *argv[],
       /* Now find the argument marker and replace by the pipe's fd.
          Yeah, that is an ugly non-thread safe hack but it safes us to
          create a copy of the array.  */
+#ifdef HAVE_W32_SYSTEM
+      snprintf (extrafdbuf, sizeof extrafdbuf, "-&%lu",
+                (unsigned long)(void*)_get_osfhandle (extrapipe[0]));
+#else
       snprintf (extrafdbuf, sizeof extrafdbuf, "-&%d", extrapipe[0]);
+#endif
       for (argsaveidx=0; argv[argsaveidx]; argsaveidx++)
         if (!strcmp (argv[argsaveidx], "-&@INEXTRA@"))
           {
@@ -442,6 +459,8 @@ gnupg_exec_tool_stream (const char *pgmname, const char *argv[],
           if (es_feof (input))
             {
               err = copy_buffer_flush (cpbuf_in, fds[0].stream);
+              if (gpg_err_code (err) == GPG_ERR_EAGAIN)
+                continue;	/* Retry next time.  */
               if (err)
                 {
                   log_error ("error feeding data to '%s': %s\n",
@@ -468,6 +487,8 @@ gnupg_exec_tool_stream (const char *pgmname, const char *argv[],
           if (es_feof (inextra))
             {
               err = copy_buffer_flush (cpbuf_extra, fds[3].stream);
+              if (gpg_err_code (err) == GPG_ERR_EAGAIN)
+                continue;	/* Retry next time.  */
               if (err)
                 {
                   log_error ("error feeding data to '%s': %s\n",
@@ -604,9 +625,11 @@ gnupg_exec_tool (const char *pgmname, const char *argv[],
 
   if (len)
     {
-      err = es_read (output, *result, len, &nread);
-      if (err)
-        goto leave;
+      if (es_read (output, *result, len, &nread))
+        {
+          err = my_error_from_syserror ();
+          goto leave;
+        }
       if (nread != len)
         log_fatal ("%s: short read from memstream\n", __func__);
     }
