@@ -14,7 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -32,9 +32,9 @@
 
 #include "g13-syshelp.h"
 #include <assuan.h>
-#include "i18n.h"
+#include "../common/i18n.h"
 #include "g13tuple.h"
-#include "exectool.h"
+#include "../common/exectool.h"
 #include "keyblob.h"
 
 /* The standard disk block size (logical).  */
@@ -55,7 +55,7 @@
 
 /* The length in blocks of the space we put at the start and at the
    end of the device.  This space is used to store N copies of the
-   setup area for the actual encrypted container inbetween.  */
+   setup area for the actual encrypted container in between.  */
 #define HEADER_SECTORS (SETUP_AREA_SECTORS * HEADER_SETUP_AREA_COPIES)
 #define FOOTER_SECTORS (SETUP_AREA_SECTORS * FOOTER_SETUP_AREA_COPIES)
 
