@@ -39,19 +39,21 @@
 				   (getenv "LD_LIBRARY_PATH"))
 		    (path-join gpgme-builddir "src/.libs"))
 		#t)
-	(call-with-fds
-	 `("/usr/bin/python"
-	   ,(in-gpgme-srcdir "lang" "python" "tests" "run-tests.py")
-	   --quiet
-	   --interpreters=/usr/bin/python
-	   --builddir ,(path-join gpgme-builddir "lang" "python" "tests")
-	   ,@what)
-	 STDIN_FILENO STDOUT_FILENO STDERR_FILENO))
-      (if #f 77 (call-with-fds what STDIN_FILENO STDOUT_FILENO STDERR_FILENO))))
+	(if python
+	    (call-with-fds
+	     `(,python
+	       ,(in-gpgme-srcdir "lang" "python" "tests" "run-tests.py")
+	       --quiet
+	       ,(string-append "--interpreters=" python)
+	       --builddir ,(path-join gpgme-builddir "lang" "python" "tests")
+	       ,@what)
+	     STDIN_FILENO STDOUT_FILENO STDERR_FILENO)
+	    77))
+      (call-with-fds what STDIN_FILENO STDOUT_FILENO STDERR_FILENO)))
 
 (let ((name (basename (car executable))))
   (cond
-   ((string=? "t-keylist" name)
+   ((string=? (qualify "t-keylist") name)
     ;; This test assumes that 't-import' imported a key.
     (log "Importing extra key...")
     (call-check `(,@GPG --yes --import ,(in-srcdir "pubkey-1.asc"))))))
