@@ -14,7 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -40,8 +40,8 @@
 #endif
 
 #define INCLUDED_BY_MAIN_MODULE 1
-#include "util.h"
-#include "openpgpdefs.h"
+#include "../common/util.h"
+#include "../common/openpgpdefs.h"
 
 static int opt_verbose;
 static const char *opt_prefix = "";
@@ -114,7 +114,7 @@ main (int argc, char **argv)
   setmode( fileno(stdin), O_BINARY );
   setmode( fileno(stdout), O_BINARY );
 #endif
-  log_set_prefix ("gpgsplit", JNLIB_LOG_WITH_PREFIX);
+  log_set_prefix ("gpgsplit", GPGRT_LOG_WITH_PREFIX);
   set_strusage (my_strusage);
 
   pargs.argc = &argc;
@@ -577,7 +577,7 @@ write_part (FILE *fpin, unsigned long pktlen,
       len = public_key_length (blob, pktlen);
       if (!len)
         {
-          log_error ("error calcualting public key length\n");
+          log_error ("error calculating public key length\n");
           g10_exit (1);
         }
       if ( (hdr[0] & 0x40) )
